//------------------------------------------------------------------------------------------------------
//	$Id: euconsent.js,v 1.4 2022/10/15 10:44:14 wolf Exp $
//------------------------------------------------------------------------------------------------------
//	Copyright (c) 2022 Wolfgang Stanglmeier <software@netzwolf.info>
//	This program is free software. It comes without any warranty.
//	You can redistribute it and/or modify it under the terms of the
//	"Do What The Fuck You Want To Public License" (WTFPL) Version 2,
//	as published by Sam Hocevar. See http://www.wtfpl.net/
//------------------------------------------------------------------------------------------------------
//	implements eu cookie consent using sessionStore or localStore
//------------------------------------------------------------------------------------------------------

/*jsl:option explicit*/

'use strict';

({
	//----------------------------------------------------------------------------------------------
	//	config
	//----------------------------------------------------------------------------------------------

	verbosity: 0,
	cookieLifetime: 0,
	storeKey: 'euconsent',
	dntPolicy: null,

	//----------------------------------------------------------------------------------------------
	//	extracted from elements
	//----------------------------------------------------------------------------------------------

	elements: null,
	loadElement: null,

	//----------------------------------------------------------------------------------------------
	//	state
	//----------------------------------------------------------------------------------------------

	consent: null,

	//----------------------------------------------------------------------------------------------
	//	gather elements and attach callback functions
	//----------------------------------------------------------------------------------------------

	init: function(){

		var closure = this;

		//--------------------------------------------------------------------------------------
		//	reset
		//--------------------------------------------------------------------------------------

		this.elements=[];

		//--------------------------------------------------------------------------------------
		//	process elements marked by "class = euconsent"
		//--------------------------------------------------------------------------------------

		var elements = document.getElementsByClassName( 'euconsent' );

		for( var e=0 ; e < elements.length ; ++e ){

			var element = elements[e];

			//------------------------------------------------------------------------------
			//	elements updated on consent change
			//------------------------------------------------------------------------------

			this.elements.push( element );

			//------------------------------------------------------------------------------
			//	callback for <button>
			//------------------------------------------------------------------------------

			if( element.tagName == 'BUTTON' ){

				element.onclick = element.ontouch = function(){

					closure.callback( this );
					return false;
				};
			}

			//------------------------------------------------------------------------------
			//	callback for <input>
			//------------------------------------------------------------------------------

			if( element.tagName == 'INPUT' ){

				element.onchange = function(){

					closure.callback( this );
					return false;
				};
			}

			//------------------------------------------------------------------------------
			//	config
			//------------------------------------------------------------------------------

			this.cookieLifetime = parseFloat( element.getAttribute(
					'data-euconsent-cookie-lifetime' )) || this.cookieLifetime;

			this.verbosity = parseFloat( element.getAttribute(
					'data-euconsent-verbosity' )) || this.verbosity;

			this.storeKey = element.getAttribute( 'data-euconsent-store-key' ) ||
								this.storeKey;

			this.dntPolicy = element.getAttribute( 'data-euconsent-dnt-policy' ) ||
								this.dntPolicy;

			//------------------------------------------------------------------------------
			//	load when needed
			//------------------------------------------------------------------------------

			if( element.hasAttribute( 'data-euconsent-consent-form-url' ) ){

				this.loadElement = element;
			}
		}

		//--------------------------------------------------------------------------------------
		//	process DNT
		//--------------------------------------------------------------------------------------

		if( this.dntPolicy && ( navigator.doNotTrack || window.doNotTrack ) ){

			this.setConsent( '' );
			this.saveConsent();

			if( this.verbosity ){

				 console.debug( 'DNT: "' + this.dntPolicy + '"' );
			}

			this.postConsent( this.dntPolicy );
			return true;
		}

		//--------------------------------------------------------------------------------------
		//	DNT not enabled or no DNT header
		//--------------------------------------------------------------------------------------

		this.loadConsent();
		this.update( true );

		if( this.consent ){

			this.postConsent( this.consent );
		} else {

			this.loadConsentForm( this.loadElement );
		}
		return true;
	},

	//----------------------------------------------------------------------------------------------
	//	process click on button or radio button
	//----------------------------------------------------------------------------------------------

	callback: function( element ){

		//--------------------------------------------------------------------------------------
		//	<button onclick=>
		//--------------------------------------------------------------------------------------

		if( element.tagName == 'BUTTON' ){

			if( element.type == 'reset' ){

				this.setConsent( '' );
				this.saveConsent();
				this.update( false );
				if( this.verbosity ){
					console.debug( 'consent: window.location.reload()' );
				}
				window.location.reload();
				return;
			}

			if( element.type == 'submit' ){

				this.saveConsent();
				this.postConsent( this.consent );
				this.update( true );
				return;
			}

			if( element.hasAttribute( 'data-euconsent-select' ) ){

				this.setConsent( element.getAttribute( 'data-euconsent-select' ));
			}
			return;
		}

		//--------------------------------------------------------------------------------------
		//	<input onchange=>
		//--------------------------------------------------------------------------------------

		if( element.tagName == 'INPUT' && element.type == 'radio' ){

			this.setConsent( element.value );
		}
	},

	//----------------------------------------------------------------------------------------------
	//	set consent
	//----------------------------------------------------------------------------------------------

	setConsent: function( new_consent ){

		this.consent = new_consent;
		if( this.verbosity ){  console.debug( 'consent := "' + this.consent + '"' ); }
		this.update( false );
	},

	//----------------------------------------------------------------------------------------------
	//	update elements dependant on consent
	//----------------------------------------------------------------------------------------------

	update: function( saved ){

		//--------------------------------------------------------------------------------------
		//	update selected / unselected - status
		//--------------------------------------------------------------------------------------

		for( var e=0 ; e < this.elements.length ; ++e ){

			var element = this.elements[e];

			//------------------------------------------------------------------------------
			//	unhide element
			//------------------------------------------------------------------------------

			if( element.style.display == 'none' ){ element.style.display = null; }

			//------------------------------------------------------------------------------
			//	selection per "class="
			//------------------------------------------------------------------------------

			if( element.hasAttribute( 'data-euconsent-select' ) ){

				if( element.getAttribute( 'data-euconsent-select' ) === this.consent ){

					element.classList.add(    'euconsent-selected'   );
					element.classList.remove( 'euconsent-unselected' );

				} else {

					element.classList.remove( 'euconsent-selected'   );
					element.classList.add(    'euconsent-unselected' );
				}

			} else if( saved ){

				if( this.consent ){

					element.classList.add(    'euconsent-selected'   );
					element.classList.remove( 'euconsent-unselected' );

				} else {

					element.classList.remove( 'euconsent-selected'   );
					element.classList.add(    'euconsent-unselected' );
				}
			}

			//------------------------------------------------------------------------------
			//	selection per "checked="
			//------------------------------------------------------------------------------

			if( element.tagName == 'INPUT' && element.type == 'radio' ){

				element.checked = element.value == this.consent;
			}

			//------------------------------------------------------------------------------
			//	disable per "disabled="
			//------------------------------------------------------------------------------

			if( element.tagName == 'BUTTON' && element.type == 'submit' ){

				element.disabled = !this.consent;
			}
		}
	},

	//----------------------------------------------------------------------------------------------
	//	post consent to business logik
	//----------------------------------------------------------------------------------------------

	postConsent: function( consent ){

		if( this.verbosity ){ console.debug( 'postConsent(' + consent + ')' ); }

		//--------------------------------------------------------------------------------------
		//	gather script urls
		//--------------------------------------------------------------------------------------

		var attribute_name = 'data-euconsent-script-urls-' + consent;
		var urls = [];

		for( var e=0 ; e < this.elements.length ; ++e ){

			var attribute = this.elements[e].getAttribute( attribute_name );
			if( !attribute ){ continue; }

			urls = urls.concat( attribute.trim().split( /\s+/ ));
		}

		//--------------------------------------------------------------------------------------
		//	load scripts
		//--------------------------------------------------------------------------------------

		for( var u=0 ; u < urls.length ; ++u ){

			var url = urls[ u ];

			if( this.verbosity ){

				console.debug( 'add <script src="' + url + '"></script>' );
			}

			var scriptElement = window.document.createElement( 'script' );
			scriptElement.setAttribute( 'src',  url );
			scriptElement.setAttribute( 'async', true );
			window.document.head.appendChild( scriptElement );
		}
	},

	//----------------------------------------------------------------------------------------------
	//	save consent
	//----------------------------------------------------------------------------------------------
	//	using localStorage if cookieLifetime > 0
	//		[consent] \x01 [set-time] \x01 [valid-seconds]
	//	using window.sessionStorage if cookieLifetime <= 0
	//		[consent]
	//----------------------------------------------------------------------------------------------

	saveConsent: function( ){

		if( this.verbosity ){

			console.debug( 'save (cookieLifetime: ' + this.cookieLifetime + ')' );
		}

		//--------------------------------------------------------------------------------------
		//	use sessionStorage
		//--------------------------------------------------------------------------------------

		if( this.cookieLifetime <= 0 ){

			if( !window.sessionStorage ){ return false; }

			if( this.consent ){

				if( this.verbosity ){

	console.debug( 'sessionStorage[' + this.storeKey + '] := "' + this.consent + '"' );
				}
				window.sessionStorage.setItem( this.storeKey, this.consent );

			} else {
				if( this.verbosity ){

	console.debug( 'sessionStorage[' + this.storeKey + '] removed.' );
				}
				window.sessionStorage.removeItem( this.storeKey );
			}
			return true;
		}

		//--------------------------------------------------------------------------------------
		//	use localStorage
		//--------------------------------------------------------------------------------------

		if( !window.localStorage ){ return false; }

		//--------------------------------------------------------------------------------------
		//	no consent set -> remove item
		//--------------------------------------------------------------------------------------

		if( !this.consent ){

			if( this.verbosity ) console.debug( 'localStorage[' + this.storeKey + '] removed.' );
			window.localStorage.removeItem( this.storeKey );
			return true;
		}

		//--------------------------------------------------------------------------------------
		//	store consent and time
		//--------------------------------------------------------------------------------------

		var value = [ this.consent, new Date().getTime(), this.cookieLifetime ].join( '\x01' );

		if( this.verbosity ){

			console.debug( 'localStorage[' + this.storeKey + '] := "' + value + '"' );
		}

		window.localStorage.setItem( this.storeKey, value );
		return true;
	},

	//----------------------------------------------------------------------------------------------
	//	load consent
	//----------------------------------------------------------------------------------------------

	loadConsent: function( ){

		if( this.verbosity ){

			console.debug( 'load (cookieLifetime: ' + this.cookieLifetime + ')' );
		}

		this.consent = '';

		//--------------------------------------------------------------------------------------
		//	use sessionStorage
		//--------------------------------------------------------------------------------------

		if( this.cookieLifetime <= 0 ){

			if( !window.sessionStorage ){ return false; }

			this.consent = window.sessionStorage.getItem( this.storeKey ) || '';
			if( this.verbosity ){

	console.debug( 'sessionStorage[' + this.storeKey + '] \u2192 "' + this.consent + '"' );
			}
			return true;
		}

		//--------------------------------------------------------------------------------------
		//	use localStorage
		//--------------------------------------------------------------------------------------

		if( !window.localStorage ){ return false; }

		var item = window.localStorage.getItem( this.storeKey );

		if( this.verbosity ){

			console.debug( 'localStorage[' + this.storeKey + '] \u2192 "' + item + '"' );
		}

		if( item === null ){

			if( this.verbosity ){

				console.debug( 'localStorage[' + this.storeKey + '] unset' );
			}
			return true;
		}

		//--------------------------------------------------------------------------------------
		//	split item into consent, time_set and expires
		//--------------------------------------------------------------------------------------

		var consent_settime_duration = item.split( '\x01' );

		var set_time        = parseFloat( consent_settime_duration[1] ) || 0;
		var expires_seconds = parseFloat( consent_settime_duration[2] ) || 0;

		//--------------------------------------------------------------------------------------
		//	check if expired
		//--------------------------------------------------------------------------------------

		var expires = set_time + Math.min( this.cookieLifetime, expires_seconds) * 1e3;
		var ttl = expires - new Date().getTime();

		if( ttl <= 0 ){

			if( this.verbosity ){

				console.debug( 'localStorage[' + this.storeKey + '] expired.' );
			}

			window.localStorage.removeItem( this.storeKey );
			return true;
		}

		this.consent = consent_settime_duration[0];

		if( this.verbosity ){

	console.debug( 'localStorage[' + this.storeKey + '] \u2192 "' + this.consent + '"' +
				' (expires in ' + ttl/1e3 + ' seconds: ' + new Date(expires) + ')' );
		}
		return true;
	},

	//----------------------------------------------------------------------------------------------
	//	dynamically load consent form
	//----------------------------------------------------------------------------------------------

	loadConsentForm: function( element ){

		if( !element ){ return false; }

		//--------------------------------------------------------------------------------------
		//	url
		//--------------------------------------------------------------------------------------

		var url = element.getAttribute( 'data-euconsent-consent-form-url' );
		if( !url ){ return false; }

		element.removeAttribute( 'data-euconsent-consent-form-url' );

		//--------------------------------------------------------------------------------------
		//	request object
		//--------------------------------------------------------------------------------------

		var request = this.createXMLHttpRequest();
		if( !request ){ element.style.display = 'none'; return false; }

		//--------------------------------------------------------------------------------------
		//	process response
		//--------------------------------------------------------------------------------------

		var closure = this;

		request.onload = function( evt ) {

			if (this.status >= 200 || this.status < 300) {

				if( closure.verbosity ){

console.debug( 'XMLHttpRequest("' + url + '") success: ' + this.responseText.length + ' bytes.' );
				}

				element.innerHTML = this.responseText;
				closure.init();
				closure.update( true );

			} else if( closure.verbosity ){

console.debug( 'XMLHttpRequest("' + url + '") returned ' + this.status + '.' );
			}
		};

		if( closure.verbosity ){ console.debug( 'XMLHttpRequest("' + url + '") \u2026' ); }

		request.open( 'GET', url );
		request.send( null );
		return true;
	},

	createXMLHttpRequest: function( ){

		// Firefox, Chromium, Opera, Safari sowie Internet Explorer (ab v7)
		try {
			return new XMLHttpRequest();
		} catch(e) {}

		// MS Internet Explorer (ab v6)
		try {
			return ActiveXObject( 'Microsoft.XMLHTTP' );
		} catch(e) {}

		// MS Internet Explorer (ab v5)
		try {
			return ActiveXObject( 'Msxml2.XMLHTTP' );
		} catch(e) {}

		return null;
	}
}).init();

//------------------------------------------------------------------------------------------------------
//	$Id: euconsent.js,v 1.4 2022/10/15 10:44:14 wolf Exp $
//------------------------------------------------------------------------------------------------------
